# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_reports', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='report',
            name='description',
            field=models.CharField(default='-', max_length=500, verbose_name='Name'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='report',
            name='style',
            field=models.CharField(default='pie', max_length=10, verbose_name='Name'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='report',
            name='title',
            field=models.CharField(default='-', max_length=100, verbose_name='Name'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='report',
            name='name',
            field=models.CharField(unique=True, max_length=10, verbose_name='Name'),
        ),
    ]



import time

from django.core.urlresolvers import reverse
from django.test import TestCase

from django_reports.models import Report


class ReportTest(TestCase):

    def setUp(self):
        # create the main workflow object
        r1 = Report.objects.create(name="Test Report 1", source_code="""
from django_reports.models import Report
from django_reports.highcharts import PieChartReportQuery

class TestQuery1(PieChartReportQuery):
        
    def get_series_data(self, **kwargs):
        return [{"name":r.name, "y":1} for r in Report.objects.all()]
    
    def get_series_name(self, **kwargs):
        return "Reports"
    
    def get_title(self, **kwargs):
        return "Reports"
    
    def get_form(parameter_name):
        pass

query = TestQuery1()
"""
        )

    def test_reports(self):
        r = Report.objects.get(name="Test Report 1")
        r.compile()
        print(r.eval())

    def test_call_view_loads(self):
        response = self.client.get(reverse("report_view"))
        print(response)
        self.assertEqual(response.status_code, 200)
        response = self.client.get(reverse("report_view"), data={"get":"values", "report": "Test Report 1"})
        print(response)
        self.assertEqual(response.status_code, 200)




from abc import ABCMeta, abstractmethod

from django_reports.reports import ReportQuery
import json

class PieChartReportQuery(ReportQuery):

    __metaclass__ = ABCMeta

    @abstractmethod
    def get_series_data(self, **kwargs):
        pass

    @abstractmethod
    def get_series_name(self, **kwargs):
        pass

    @abstractmethod
    def get_title(self, **kwargs):
        pass

    def eval(self, **kwargs):
        return {
            "chart": {
                "plotBackgroundColor": None,
                "plotBorderWidth": None,
                "plotShadow": False,
                "type": "pie"
            },
            "title": {
                "text": self.get_title(**kwargs)
            },
            "tooltip": {
                "pointFormat": "{series.name}: <b>{point.y:.1f}%</b>"
            },
            "plotOptions": {
                "pie": {
                    "allowPointSelect": True,
                    "cursor": 'pointer',
                    "dataLabels": {
                        "enabled": True,
                        "format": "<b>{point.name}</b>: {point.y:.1f} %",
                        "style": {
                            "color": "black"
                        }
                    }
                }
            },
            "series": [{
                "name": self.get_series_name(**kwargs),
                "colorByPoint": True,
                "data": self.get_series_data(**kwargs)
            }]

        }

class BarChartReportQuery(ReportQuery):

    __metaclass__ = ABCMeta

    ## returns an array or tuple of series labels
    @abstractmethod
    def get_series_names(self, **kwargs):
        return ["series 1", "series 2"]

    ## returns the data for the specified series as array of values
    @abstractmethod
    def get_series_data(self, series, **kwargs):
        return [1.0, 2.1, 1.5, 0.4]

    @abstractmethod
    def get_x_labels(self, **kwargs):
        return ["May", "June", "July", "August"]

    @abstractmethod
    def get_title(self, **kwargs):
        return "Bar Chart"

    @abstractmethod
    def get_y_title(self, **kwargs):
        return "Average"

    def eval(self, **kwargs):
        return {
            "chart": {
                "type": "column"
            },
            "title": {
                "text": self.get_title(**kwargs)
            },
            "xAxis": {
                "categories": self.get_x_labels(**kwargs),
            },
            "yAxis": {
                "min": 0,
                "title": {
                    "text": self.get_y_title(**kwargs)
                },
                "stackLabels": {
                    "enabled": True,
                    "style": {
                        "fontWeight": "bold",
                        "color2":  "gray"
                    }
                }
            },
            "legend": {
                "align": "right",
                "x": -30,
                "verticalAlign": "top",
                "y": 25,
                "floating": True,
                "backgroundColor": "white",
                "borderColor": "#CCC",
                "borderWidth": 1,
                "shadow": False
            },

            "tooltip": {
                "headerFormat": "<b>{point.x}</b><br/>",
                "pointFormat": "{series.name}: {point.y}<br/>Total: {point.stackTotal}"
            },
            "plotOptions": {
                "column": {
                    "stacking": 'normal',
                    "dataLabels": {
                        "enabled": True,
                        "color": "white"
                    }
                }
            },
            "series":
            [{
                "name": s,
                "colorByPoint": True,
                "data": self.get_series_data(s, **kwargs)
            } for s in self.get_series_names(**kwargs)]

        }